"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var react_1 = tslib_1.__importDefault(require("react"));
var enzyme_adapter_react_16_1 = tslib_1.__importDefault(require("enzyme-adapter-react-16"));
var testComponents_1 = require("./testComponents");
var enzyme_1 = require("enzyme");
var LayoutRegistration_1 = require("../LayoutRegistration");
var get_registration_creators_1 = require("../get-registration-creators");
enzyme_1.configure({ adapter: new enzyme_adapter_react_16_1.default() });
var _a = get_registration_creators_1.getRegistrationCreators(), createRegisterableComponent = _a.createRegisterableComponent, createRegisterableComposition = _a.createRegisterableComposition;
it('can hook component middleware', function () {
    var middlewareCalled;
    var middlewareProps;
    var layout = LayoutRegistration_1.LayoutRegistration()
        .registerComponents(function (registrar) {
        return registrar
            .registerComponent(testComponents_1.testComponentWithPropsRegistration)
            .registerMiddleware(function (cp, mp, ms, next) {
            middlewareCalled = true;
            middlewareProps = mp;
            return next(cp, mp, ms);
        });
    })
        .registerCompositions(function (registrar) {
        return registrar.registerComposition(testComponents_1.testCompositionWithPropsRegistration);
    });
    var renderer = layout.createRenderers({ services: {} });
    var renderOutput = enzyme_1.mount(renderer.renderComponents(layout.component({
        type: 'testWithTitleProp',
        props: { title: 'test' },
        skipRender: true,
    })));
    expect(middlewareCalled).toBe(true);
    expect(middlewareProps).toMatchObject({ skipRender: true });
    expect(renderOutput.find(testComponents_1.TestComponentWithProps).length).toBe(1);
    expect(renderOutput.text()).toContain('test');
});
it('can hook multiple component middleware', function () {
    var middlewareCalled;
    var middleware2Called;
    var middleware2Props;
    var middleware2ComponentProps;
    var middleware2Services;
    var componenentServices;
    var layout = LayoutRegistration_1.LayoutRegistration()
        .registerComponents(function (registrar) {
        return registrar
            .registerComponent(createRegisterableComponent('testWithTitleProp', function (props, services) {
            componenentServices = services;
            return react_1.default.createElement(testComponents_1.TestComponentWithProps, { title: props.title });
        }))
            .registerMiddleware(function (props, _, services, next) {
            middlewareCalled = true;
            if (middleware2Called) {
                throw new Error('middlewares called out of order');
            }
            return next(tslib_1.__assign(tslib_1.__assign({}, props), { additional: true }), _, tslib_1.__assign(tslib_1.__assign({}, services), { services: { serviceValue: true } }));
        })
            .registerMiddleware(function (componentProps, middlewareProps, services, next) {
            middleware2Called = true;
            middleware2ComponentProps = componentProps;
            middleware2Props = middlewareProps;
            middleware2Services = services;
            return next(middleware2ComponentProps, middleware2Props, middleware2Services);
        });
    })
        .registerCompositions(function (registrar) {
        return registrar.registerComposition(testComponents_1.testCompositionWithPropsRegistration);
    });
    var renderers = layout.createRenderers({ services: { serviceValue: false } });
    var renderOutput1 = enzyme_1.mount(renderers.renderComponents(layout.component({
        type: 'testWithTitleProp',
        props: { title: 'test' },
        skipRender2: true,
    })));
    expect(middlewareCalled).toBe(true);
    expect(middleware2Called).toBe(true);
    expect(middleware2Props).toMatchObject({ skipRender2: true });
    expect(middleware2ComponentProps).toMatchObject({ additional: true });
    expect(middleware2Services).toMatchObject({ services: { serviceValue: true } });
    expect(componenentServices).toMatchObject({ serviceValue: true });
    expect(renderOutput1.find(testComponents_1.TestComponentWithProps).length).toBe(1);
    expect(renderOutput1.text()).toContain('test');
});
it('can hook composition middleware', function () {
    var middlewareCalled;
    var middlewareProps;
    var middlewareNext;
    var layout = LayoutRegistration_1.LayoutRegistration()
        .registerComponents(function (registrar) {
        return registrar.registerComponent(testComponents_1.testComponentWithPropsRegistration);
    })
        .registerCompositions(function (registrar) {
        return registrar
            .registerComposition(testComponents_1.testCompositionWithPropsRegistration)
            .registerMiddleware(function (_, mp, __, next) {
            middlewareCalled = true;
            middlewareProps = mp;
            middlewareNext = next;
            return null;
        });
    });
    var renderers = layout.createRenderers({ services: {} });
    enzyme_1.mount(renderers.renderCompositions(layout.composition({
        type: 'test-composition-with-props',
        contentAreas: {
            main: [
                layout.component({
                    type: 'testWithTitleProp',
                    props: { title: 'Component title' },
                }),
            ],
        },
        props: { compositionTitle: 'Composition title' },
        skipRender: true,
    })));
    expect(middlewareCalled).toBe(true);
    expect(middlewareProps).toMatchObject({ skipRender: true });
    // Verify next() will actually render the component
    var renderOutput = enzyme_1.mount(middlewareNext());
    expect(renderOutput.find(testComponents_1.TestComponentWithProps).length).toBe(1);
    expect(renderOutput.text()).toContain('Component title');
    expect(renderOutput.text()).toContain('Composition title');
});
it('can hook multiple composition middleware', function () {
    var middlewareCalled;
    var middleware2Called;
    var middleware2Props;
    var middleware2ComponentProps;
    var middleware2Next;
    var middleware2Service;
    var layout = LayoutRegistration_1.LayoutRegistration()
        .registerComponents(function (registrar) {
        return registrar.registerComponent(testComponents_1.testComponentWithPropsRegistration);
    })
        .registerCompositions(function (registrar) {
        return registrar
            .registerComposition(testComponents_1.testCompositionWithPropsRegistration)
            .registerMiddleware(function (props, _, services, next) {
            middlewareCalled = true;
            if (middleware2Called) {
                throw new Error('middlewares called out of order');
            }
            return next(tslib_1.__assign(tslib_1.__assign({}, props), { additional: true }), _, tslib_1.__assign(tslib_1.__assign({}, services), { services: { serviceAvailable: true } }));
        })
            .registerMiddleware(function (componentProps, middlewareProps, service, next) {
            middleware2Called = true;
            middleware2ComponentProps = componentProps;
            middleware2Props = middlewareProps;
            middleware2Next = next;
            middleware2Service = service;
            return null;
        });
    });
    var renderers = layout.createRenderers({ services: { serviceAvailable: false } });
    enzyme_1.mount(renderers.renderCompositions(layout.composition({
        type: 'test-composition-with-props',
        contentAreas: {
            main: [
                layout.component({
                    type: 'testWithTitleProp',
                    props: { title: 'Component title' },
                }),
            ],
        },
        props: { compositionTitle: 'Composition title' },
        skipRender2: true,
    })));
    expect(middlewareCalled).toBe(true);
    expect(middleware2Called).toBe(true);
    expect(middleware2Props).toMatchObject({ skipRender2: true });
    expect(middleware2ComponentProps).toMatchObject({ additional: true });
    expect(middleware2Service).toMatchObject({ services: { serviceAvailable: true } });
    // Verify next() will actually render the component
    var renderOutput1 = enzyme_1.mount(middleware2Next());
    expect(renderOutput1.find(testComponents_1.TestComponentWithProps).length).toBe(1);
    expect(renderOutput1.text()).toContain('Composition title');
    // Verify next() will actually render the component
    var renderOutput = enzyme_1.mount(middleware2Next({ compositionTitle: 'Override' }));
    expect(renderOutput.find(testComponents_1.TestComponentWithProps).length).toBe(1);
    expect(renderOutput.text()).toContain('Override');
});
//# sourceMappingURL=middleware-support.test.js.map