import { __assign } from "tslib";
import React from 'react';
import Adapter from 'enzyme-adapter-react-16';
import { testComponentWithPropsRegistration, TestComponentWithProps, testCompositionWithPropsRegistration, } from './testComponents';
import { configure, mount } from 'enzyme';
import { LayoutRegistration } from '../LayoutRegistration';
import { getRegistrationCreators } from '../get-registration-creators';
configure({ adapter: new Adapter() });
var _a = getRegistrationCreators(), createRegisterableComponent = _a.createRegisterableComponent, createRegisterableComposition = _a.createRegisterableComposition;
it('can hook component middleware', function () {
    var middlewareCalled;
    var middlewareProps;
    var layout = LayoutRegistration()
        .registerComponents(function (registrar) {
        return registrar
            .registerComponent(testComponentWithPropsRegistration)
            .registerMiddleware(function (cp, mp, ms, next) {
            middlewareCalled = true;
            middlewareProps = mp;
            return next(cp, mp, ms);
        });
    })
        .registerCompositions(function (registrar) {
        return registrar.registerComposition(testCompositionWithPropsRegistration);
    });
    var renderer = layout.createRenderers({ services: {} });
    var renderOutput = mount(renderer.renderComponents(layout.component({
        type: 'testWithTitleProp',
        props: { title: 'test' },
        skipRender: true,
    })));
    expect(middlewareCalled).toBe(true);
    expect(middlewareProps).toMatchObject({ skipRender: true });
    expect(renderOutput.find(TestComponentWithProps).length).toBe(1);
    expect(renderOutput.text()).toContain('test');
});
it('can hook multiple component middleware', function () {
    var middlewareCalled;
    var middleware2Called;
    var middleware2Props;
    var middleware2ComponentProps;
    var middleware2Services;
    var componenentServices;
    var layout = LayoutRegistration()
        .registerComponents(function (registrar) {
        return registrar
            .registerComponent(createRegisterableComponent('testWithTitleProp', function (props, services) {
            componenentServices = services;
            return React.createElement(TestComponentWithProps, { title: props.title });
        }))
            .registerMiddleware(function (props, _, services, next) {
            middlewareCalled = true;
            if (middleware2Called) {
                throw new Error('middlewares called out of order');
            }
            return next(__assign(__assign({}, props), { additional: true }), _, __assign(__assign({}, services), { services: { serviceValue: true } }));
        })
            .registerMiddleware(function (componentProps, middlewareProps, services, next) {
            middleware2Called = true;
            middleware2ComponentProps = componentProps;
            middleware2Props = middlewareProps;
            middleware2Services = services;
            return next(middleware2ComponentProps, middleware2Props, middleware2Services);
        });
    })
        .registerCompositions(function (registrar) {
        return registrar.registerComposition(testCompositionWithPropsRegistration);
    });
    var renderers = layout.createRenderers({ services: { serviceValue: false } });
    var renderOutput1 = mount(renderers.renderComponents(layout.component({
        type: 'testWithTitleProp',
        props: { title: 'test' },
        skipRender2: true,
    })));
    expect(middlewareCalled).toBe(true);
    expect(middleware2Called).toBe(true);
    expect(middleware2Props).toMatchObject({ skipRender2: true });
    expect(middleware2ComponentProps).toMatchObject({ additional: true });
    expect(middleware2Services).toMatchObject({ services: { serviceValue: true } });
    expect(componenentServices).toMatchObject({ serviceValue: true });
    expect(renderOutput1.find(TestComponentWithProps).length).toBe(1);
    expect(renderOutput1.text()).toContain('test');
});
it('can hook composition middleware', function () {
    var middlewareCalled;
    var middlewareProps;
    var middlewareNext;
    var layout = LayoutRegistration()
        .registerComponents(function (registrar) {
        return registrar.registerComponent(testComponentWithPropsRegistration);
    })
        .registerCompositions(function (registrar) {
        return registrar
            .registerComposition(testCompositionWithPropsRegistration)
            .registerMiddleware(function (_, mp, __, next) {
            middlewareCalled = true;
            middlewareProps = mp;
            middlewareNext = next;
            return null;
        });
    });
    var renderers = layout.createRenderers({ services: {} });
    mount(renderers.renderCompositions(layout.composition({
        type: 'test-composition-with-props',
        contentAreas: {
            main: [
                layout.component({
                    type: 'testWithTitleProp',
                    props: { title: 'Component title' },
                }),
            ],
        },
        props: { compositionTitle: 'Composition title' },
        skipRender: true,
    })));
    expect(middlewareCalled).toBe(true);
    expect(middlewareProps).toMatchObject({ skipRender: true });
    // Verify next() will actually render the component
    var renderOutput = mount(middlewareNext());
    expect(renderOutput.find(TestComponentWithProps).length).toBe(1);
    expect(renderOutput.text()).toContain('Component title');
    expect(renderOutput.text()).toContain('Composition title');
});
it('can hook multiple composition middleware', function () {
    var middlewareCalled;
    var middleware2Called;
    var middleware2Props;
    var middleware2ComponentProps;
    var middleware2Next;
    var middleware2Service;
    var layout = LayoutRegistration()
        .registerComponents(function (registrar) {
        return registrar.registerComponent(testComponentWithPropsRegistration);
    })
        .registerCompositions(function (registrar) {
        return registrar
            .registerComposition(testCompositionWithPropsRegistration)
            .registerMiddleware(function (props, _, services, next) {
            middlewareCalled = true;
            if (middleware2Called) {
                throw new Error('middlewares called out of order');
            }
            return next(__assign(__assign({}, props), { additional: true }), _, __assign(__assign({}, services), { services: { serviceAvailable: true } }));
        })
            .registerMiddleware(function (componentProps, middlewareProps, service, next) {
            middleware2Called = true;
            middleware2ComponentProps = componentProps;
            middleware2Props = middlewareProps;
            middleware2Next = next;
            middleware2Service = service;
            return null;
        });
    });
    var renderers = layout.createRenderers({ services: { serviceAvailable: false } });
    mount(renderers.renderCompositions(layout.composition({
        type: 'test-composition-with-props',
        contentAreas: {
            main: [
                layout.component({
                    type: 'testWithTitleProp',
                    props: { title: 'Component title' },
                }),
            ],
        },
        props: { compositionTitle: 'Composition title' },
        skipRender2: true,
    })));
    expect(middlewareCalled).toBe(true);
    expect(middleware2Called).toBe(true);
    expect(middleware2Props).toMatchObject({ skipRender2: true });
    expect(middleware2ComponentProps).toMatchObject({ additional: true });
    expect(middleware2Service).toMatchObject({ services: { serviceAvailable: true } });
    // Verify next() will actually render the component
    var renderOutput1 = mount(middleware2Next());
    expect(renderOutput1.find(TestComponentWithProps).length).toBe(1);
    expect(renderOutput1.text()).toContain('Composition title');
    // Verify next() will actually render the component
    var renderOutput = mount(middleware2Next({ compositionTitle: 'Override' }));
    expect(renderOutput.find(TestComponentWithProps).length).toBe(1);
    expect(renderOutput.text()).toContain('Override');
});
//# sourceMappingURL=middleware-support.test.js.map