import { __assign } from "tslib";
export function getComponentPath(options) {
    var subpath = options.subpath, index = options.index, prefix = options.prefix;
    var componentPath = index !== undefined ? "[" + index + "]" + subpath : subpath;
    var prefixPath = prefix ? "" + prefix : '';
    var renderPath = "" + prefixPath + componentPath;
    return renderPath;
}
export function isNestedComposition(component) {
    return component.type === 'nested-composition';
}
export function flatMap(collection, map) {
    return collection.reduce(function (acc, val, i) {
        map(val, i).forEach(function (v) { return acc.push(v); });
        return acc;
    }, []);
}
/** Maps an array of compositions to a flat list of components */
export function getComponentsInCompositions(compositions, componentRenderPath) {
    return flatMap(compositions, function (composition, i) {
        return flatMap(Object.keys(composition.contentAreas), function (key) {
            return contentAreas(composition, i, key, componentRenderPath !== undefined, componentRenderPath);
        });
    });
}
/** Takes an array of compositions, and expands any nested compostions
 * into a flat list of components
 */
function expandNestedCompositionsIntoComponents(components, componentRenderPath) {
    var contentAreasArr = [];
    components.forEach(function (c, i) {
        if (isNestedComposition(c)) {
            var props = c.props;
            var result = getComponentsInCompositions([props.composition], componentRenderPath + "/[" + i + "]");
            result.forEach(function (col) { return contentAreasArr.push(col); });
        }
        else {
            contentAreasArr.push(__assign(__assign({}, c), { componentRenderPath: componentRenderPath + "/[" + i + "]" }));
        }
    });
    return contentAreasArr;
}
// map the content areas from the given composition
function contentAreas(composition, i, contentAreaKey, innerSearch, componentRenderPath) {
    var routeDataOptions = {
        subpath: !innerSearch ? composition.type : 'nested:' + composition.type,
        index: !innerSearch ? i : undefined,
        prefix: componentRenderPath,
    };
    var components = composition.contentAreas[contentAreaKey];
    var path = getComponentPath(routeDataOptions) + "/" + contentAreaKey;
    return expandNestedCompositionsIntoComponents(components, path);
}
//# sourceMappingURL=helpers.js.map