import { __rest } from "tslib";
import React from 'react';
import { ComponentsRenderer } from './components-renderer';
export var CompositionRenderer = function (_a) {
    var services = _a.services, layoutApi = _a.layoutApi, componentRenderPath = _a.componentRenderPath, composition = _a.composition, compositionMiddleware = _a.compositionMiddleware, componentRegistrations = _a.componentRegistrations, componentMiddleware = _a.componentMiddleware, compositionRegistrations = _a.compositionRegistrations, log = _a.log;
    /**
     * The ContentAreaRenderer componentRenderPaths need to append `/[contentArea key]'
     * key as this logic is duped outside of react for the ssr
     */
    log.debug({
        componentRenderPath: componentRenderPath,
        type: composition.type,
    }, 'Rendering composition');
    var contentAreas = composition.contentAreas, compositionProps = composition.props, middlewareProps = __rest(composition, ["contentAreas", "props"]);
    var componentServices = {
        services: services,
        layout: layoutApi,
    };
    var compositionElement = compositionRegistrations.get(composition.type);
    if (!compositionElement) {
        return null;
    }
    // A middleware may call next with props, we should use them
    function render(middlewareCompositionProps) {
        var contentAreas = Object.keys(composition.contentAreas).reduce(function (acc, val) {
            var componentsRenderPath = componentRenderPath + "/" + val;
            log.debug({
                componentsRenderPath: componentsRenderPath,
                components: composition.contentAreas[val].map(function (component) { return ({
                    type: component.type,
                }); }),
            }, 'Rendering content area');
            acc[val] = (React.createElement(ComponentsRenderer, { componentMiddleware: componentMiddleware, componentRegistrations: componentRegistrations, componentRenderPath: componentsRenderPath, components: composition.contentAreas[val], layoutApi: layoutApi, services: services }));
            return acc;
        }, {});
        var rendered = compositionElement.render(contentAreas, middlewareCompositionProps || compositionProps, services) || null;
        return rendered;
    }
    var middlewareRender = compositionMiddleware(composition.props, middlewareProps, componentServices, render) || null;
    return middlewareRender;
};
CompositionRenderer.displayName = 'CompositionRenderer';
//# sourceMappingURL=composition-renderer.js.map